import { ColumnProps, CrudExpose, PageQuery, UserPageQuery } from "../../d";
import { CsvParams, ExcelParams, ExportColumn, ExportUtil, ImportUtil } from "./lib/d";
export declare function loadFsExportUtil(): Promise<ExportUtil>;
export declare function loadFsImportUtil(): Promise<ImportUtil>;
export type DataFormatterContext<R = any> = {
    row: any;
    /**
     * 原始行数据
     */
    originalRow: R;
    key: string;
    col: ColumnProps<R>;
    exportCol: ExportColumn<R>;
};
export type ColumnBuilderContext<R = any> = {
    col: ExportColumn<R>;
};
/**
 * 导出配置
 */
export type ExportProps<R = any> = {
    /**
     * 服务端导出，自己实现
     */
    server?: (pageQuery: UserPageQuery<R>) => Promise<void>;
    /**
     * 仅导出显示的列
     */
    onlyShow?: boolean;
    /**
     * 列过滤器
     * @param col
     */
    columnFilter?: (col: ColumnProps<R>) => boolean;
    /**
     * 列配置构建器
     */
    columnBuilder?: (context: ColumnBuilderContext<R>) => void;
    /**
     * 数据mapping
     */
    dataFormatter?: (context: DataFormatterContext<R>) => void;
    /**
     * 导出文件类型
     */
    fileType?: "csv" | "excel";
    /**
     * 数据来源
     * local: 本地当前页数据（默认）
     * search: 搜索数据
     */
    dataFrom?: "local" | "search";
    /**
     * 查询参数
     */
    searchParams?: PageQuery<R>;
    /**
     * 配置了dict的字段是否自动根据value获取label
     * 默认值：true
     */
    autoUseDictLabel?: boolean;
    /**
     * 数据分隔符
     */
    separator?: string;
    /**
     * 数据是否加引号
     */
    quoted?: boolean;
} & CsvParams & ExcelParams;
export declare function exportTable<R = any>(crudExpose: CrudExpose<R>, opts?: ExportProps<R>): Promise<any>;
export type ImportProps = {
    file: File;
    append?: boolean;
};
export declare function importTable<R = any>(crudExpose: CrudExpose<R>, opts: ImportProps): Promise<void>;
